unit lesson4_2_main;

{******************************************************************************}
{                                                                              }
{       Borland Delphi Runtime Library                                         }
{       Mini Variante des DGL VCL Templates, um Demos klein zu halten          }
{                                                                              }
{ Copyright (C) 2003 Nico Michaelis(Delphic)                                   }
{ All Rights Reserved.                                                         }
{                                                                              }
{       Obtained through:                                                      }
{       Delphi OpenGL Community(DGL)                                           }
{                                                                              }
{ You may retrieve the latest version of this file at the Delphi OpenGL        }
{ Community home page, located at http://www.delphigl.com/                     }
{                                                                              }
{ The contents of this file are used with permission, subject to               }
{ the Mozilla Public License Version 1.1 (the "License"); you may              }
{ not use this file except in compliance with the License. You may             }
{ obtain a copy of the License at                                              }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an                  }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or               }
{ implied. See the License for the specific language governing                 }
{ rights and limitations under the License.                                    }
{                                                                              }
{******************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OpenGl12, template_init, glaux;

const
  Projektname = 'Mini-VCL Lektion 4.2';

type
  TMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    tex : GLUint;

    x : double;
    y : double;

    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  if not LoadOpenGl then
  begin
    MessageBeep(32);
    Halt(100)
  end
end;

procedure TMainForm.FormShow(Sender: TObject);
  procedure InitTextures;
  var
    texture1: PTAUX_RGBImageRec;
  begin
    texture1 := auxDIBImageLoadA('./texture/wall.bmp');
    if not Assigned( texture1 ) then begin
      MessageBox(0,'Texturenpfad oder Dateiname falsch!'+#13+' Bitte den Pfad berprfen!','Fehler beim Laden der Textur!',MB_OK or MB_ICONERROR);
    end;

    glGenTextures(1, @tex);
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, texture1^.sizeX, texture1^.sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, texture1^.data);

  end;

begin
  //OnIdle setzen
  Application.OnIdle := ApplicationEventsIdle;
  InitGl(MainForm);
  glEnable(GL_TEXTURE_2D);
  x := 0;
  y := 0;
  InitTextures
end;

procedure TMainForm.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
var
  Error : Integer;

begin
  //Abbrechen, wenn OpenGL noch nicht initialisiert ist
  if not OpenGLInitialized then Exit;
  //Immer schn neu zeichnen
  Done := False;

 glClear(GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT);

  x:=x+0.01;
  y:=y+0.001;

    // Wir verndern unsere Texture-Matrix
  glMatrixMode(GL_TEXTURE);
    glLoadIdentity;
    glTranslatef(x,y,0);
  glMatrixMode(GL_MODELVIEW); // Nicht vergessen den Matrix-Mode wieder zurck zu schalten

  glLoadIdentity;
  gltranslatef(0,0,-5);

  glBindTexture(GL_TEXTURE_2D,tex);
  glBegin(GL_QUADS);
    glTexCoord2f(0,0); glVertex3f(-1,-1,0);
    glTexCoord2f(0,1); glVertex3f(-1,1,0);
    glTexCoord2f(1,1); glVertex3f(1,1,0);
    glTexCoord2f(1,0); glVertex3f(1,-1,0);
  glEnd;

  //Error Handler
  Error := glgetError;
  if Error <> GL_NO_ERROR then
  begin
    Caption := gluErrorString(Error);
    //Rendering kurz anhalten
    Done := True;
    FlashWindow(Handle, True)
  end;
  //Frame Counter
  Inc(Frames);
  if GetTickCount - StartTick >=500 then
  begin
    Caption := Format('http://www.delphigl.com/ - %s [%f]', [Projektname, Frames/(GetTickCount - StartTick)*1000]);
    Frames := 0;
    StartTick := GetTickCount
  end;
  //Und anzeigen
  SwapBuffers(Canvas.Handle)
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  if not OpenGlInitialized then Exit;
  glViewport(0, 0, ClientWidth, ClientHeight);
  //Projektionsmatrix resetten
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //Perspektivische Darstellung
  gluPerspective(45.0,ClientWidth/ClientHeight,NearClipping,FarClipping);// Calculate The Aspect Ratio Of The Window
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  ShutdownGl
end;

end.
