unit lesson4_4_main;

{******************************************************************************}
{                                                                              }
{       Borland Delphi Runtime Library                                         }
{       Mini Variante des DGL VCL Templates, um Demos klein zu halten          }
{                                                                              }
{ Copyright (C) 2003 Nico Michaelis(Delphic)                                   }
{ All Rights Reserved.                                                         }
{                                                                              }
{       Obtained through:                                                      }
{       Delphi OpenGL Community(DGL)                                           }
{                                                                              }
{ You may retrieve the latest version of this file at the Delphi OpenGL        }
{ Community home page, located at http://www.delphigl.com/                     }
{                                                                              }
{ The contents of this file are used with permission, subject to               }
{ the Mozilla Public License Version 1.1 (the "License"); you may              }
{ not use this file except in compliance with the License. You may             }
{ obtain a copy of the License at                                              }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an                  }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or               }
{ implied. See the License for the specific language governing                 }
{ rights and limitations under the License.                                    }
{                                                                              }
{******************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OpenGl12, template_init, glaux;

const
  Projektname = 'Mini-VCL Lektion 4.4';

  mwidth  = 30;
  mheight = 10;

type
  TMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    tex : gluInt;
    MAP : array[0..mWIDTH+1,0..mHEIGHT+1] of single;
    z : single;

    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  if not LoadOpenGl then
  begin
    MessageBeep(32);
    Halt(100)
  end
end;

procedure TMainForm.FormShow(Sender: TObject);
  procedure InitMap;
  var
    texture1: PTAUX_RGBImageRec;
    x, y : integer;
  begin
    texture1 := auxDIBImageLoadA('./texture/venus.bmp');
    if not Assigned( texture1 ) then begin
      MessageBox(0,'Texturenpfad oder Dateiname falsch!'+#13+' Bitte den Pfad berprfen!','Fehler beim Laden der Textur!',MB_OK or MB_ICONERROR);
    end;

    glGenTextures(1, @tex);
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, texture1^.sizeX, texture1^.sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, texture1^.data);

    randomize;
    for x:=0 to mWidth do
    for y:=0 to mHeight do
    Map[x,y]:=0.01*Random(50);
  end;

begin
  //OnIdle setzen
  Application.OnIdle := ApplicationEventsIdle;
  InitGl(MainForm);
  glEnable(GL_TEXTURE_2D);
  InitMap;
end;

procedure TMainForm.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
var
  Error : Integer;
  x,y: integer;
   u,v: single;

begin
  //Abbrechen, wenn OpenGL noch nicht initialisiert ist
  if not OpenGLInitialized then Exit;
  //Immer schn neu zeichnen
  Done := False;

  glClear(GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT);

  z:=z-0.01;

  glLoadIdentity;
  gluLookAt(0,2,5,
            0,0,0,
            0,1,0);
  glPushMatrix;
  gltranslatef(z,0,0);
  for y:=0 to mHeight-1 do begin
    glPushMatrix;
    for x:=0 to mWidth-1 do begin
      u:=1/mwidth;
      v:=1/mheight;
      glBegin(gl_QUADS);
        glNormal3f( 0.0, 1.0, 0.0);
        glTexCoord2f(u*x,     v*(y+1));
        glVertex3f(0,  map[x,y+1],  0);
        glTexCoord2f(u*x,     v*y);
        glVertex3f(0,  map[x,y],  1);
        glTexCoord2f(u*(x+1), v*y);
        glVertex3f(1,  map[x+1,y],  1);
        glTexCoord2f(u*(x+1), v*(y+1));
        glVertex3f(1, map[x+1,y+1],  0);
      glEnd;
      glTranslatef(1,0,0);
    end;
  glPopMatrix;
  glTranslatef(0,0,-1);
  end;
  glPopMatrix;

  //Error Handler
  Error := glgetError;
  if Error <> GL_NO_ERROR then
  begin
    Caption := gluErrorString(Error);
    //Rendering kurz anhalten
    Done := True;
    FlashWindow(Handle, True)
  end;
  //Frame Counter
  Inc(Frames);
  if GetTickCount - StartTick >=500 then
  begin
    Caption := Format('http://www.delphigl.com/ - %s [%f]', [Projektname, Frames/(GetTickCount - StartTick)*1000]);
    Frames := 0;
    StartTick := GetTickCount
  end;
  //Und anzeigen
  SwapBuffers(Canvas.Handle)
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  if not OpenGlInitialized then Exit;
  glViewport(0, 0, ClientWidth, ClientHeight);
  //Projektionsmatrix resetten
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //Perspektivische Darstellung
  gluPerspective(45.0,ClientWidth/ClientHeight,NearClipping,FarClipping);// Calculate The Aspect Ratio Of The Window
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  ShutdownGl
end;

end.
