unit lesson4_5_main;

{******************************************************************************}
{                                                                              }
{       Borland Delphi Runtime Library                                         }
{       Mini Variante des DGL VCL Templates, um Demos klein zu halten          }
{                                                                              }
{ Copyright (C) 2003 Nico Michaelis(Delphic)                                   }
{ All Rights Reserved.                                                         }
{                                                                              }
{       Obtained through:                                                      }
{       Delphi OpenGL Community(DGL)                                           }
{                                                                              }
{ You may retrieve the latest version of this file at the Delphi OpenGL        }
{ Community home page, located at http://www.delphigl.com/                     }
{                                                                              }
{ The contents of this file are used with permission, subject to               }
{ the Mozilla Public License Version 1.1 (the "License"); you may              }
{ not use this file except in compliance with the License. You may             }
{ obtain a copy of the License at                                              }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an                  }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or               }
{ implied. See the License for the specific language governing                 }
{ rights and limitations under the License.                                    }
{                                                                              }
{******************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OpenGl12, template_init, glaux;

const
  Projektname = 'Mini-VCL Lektion 4.5';

type
  TMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    tex : gluInt;
    pic : Single;

    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  if not LoadOpenGl then
  begin
    MessageBeep(32);
    Halt(100)
  end
end;

procedure TMainForm.FormShow(Sender: TObject);
  procedure InitTextures;
  var
    texture1: PTAUX_RGBImageRec;
  begin
    texture1 := auxDIBImageLoadA('./texture/numbers.bmp');
    if not Assigned( texture1 ) then begin
      MessageBox(0,'Texturenpfad oder Dateiname falsch!'+#13+' Bitte den Pfad berprfen!','Fehler beim Laden der Textur!',MB_OK or MB_ICONERROR);
    end;

    glGenTextures(1, @tex);
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, texture1^.sizeX, texture1^.sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, texture1^.data);

  end;

begin
  //OnIdle setzen
  Application.OnIdle := ApplicationEventsIdle;
  InitGl(MainForm);
  glEnable(GL_TEXTURE_2D);
  InitTextures
end;

procedure TMainForm.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
var
  Error : Integer;
  u, s : single;

begin
  //Abbrechen, wenn OpenGL noch nicht initialisiert ist
  if not OpenGLInitialized then Exit;
  //Immer schn neu zeichnen
  Done := False;

  glClear(GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT);

  glLoadIdentity;
  glTranslatef(0,-2,-10);

  glBindTexture(GL_TEXTURE_2D,tex);     // Texture setzen

  pic:=pic+0.01;      // Geschwindigkeit fr einen Bildwechsel
  s:=1/8;             { Berechnung der Bildgre. In diesem Fall besteht das Bild aus 8 Bildern,
                         beim UV-Mapping ist das Maximum 1, d.h. 1/8 = 0.125. D.h.
                         0 - 0.125      #Bild 1
                         0.125 - 0.250  #Bild 2
                         etc
                        }

  { Wir berechnen das UV-Mapping jedesmal neu. Grob behalten wir immer die Gre 1/8, scrollen,
    die Texture, aber praktisch Schritt fr Schritt, so dass diese langsam auf dem Screen
    lngs scrollt}
  glBegin(GL_QUADS);
    glTexCoord2f(pic*s,1); glVertex3f(-1,1,0);
    glTexCoord2f(pic*s+s,1); glVertex3f(1,1,0);
    glTexCoord2f(pic*s+s,0); glVertex3f(1,-1,0);
    glTexCoord2f(pic*s,0); glVertex3f(-1,-1,0);
  glEnd;

  { Ein Interessantes Beispiel dafr, das die richtigen Zahlen sehr wichtig sein
    knnen. Wir runden unsere Werte hier einfach, d.h. wir erreichen nur volle Bildteiel
    1/8*x. Da x nur ganzzahlig ist, gibt es kein flssige Animation, sondern ein "umblenden".
   }

  glTranslatef(0,3,0);
  glBegin(GL_QUADS);
    glTexCoord2f(Round(pic)*s,1); glVertex3f(-1,1,0);
    glTexCoord2f(Round(pic)*s+s,1); glVertex3f(1,1,0);
    glTexCoord2f(Round(pic)*s+s,0); glVertex3f(1,-1,0);
    glTexCoord2f(Round(pic)*s,0); glVertex3f(-1,-1,0);
  glEnd;

  //Error Handler
  Error := glgetError;
  if Error <> GL_NO_ERROR then
  begin
    Caption := gluErrorString(Error);
    //Rendering kurz anhalten
    Done := True;
    FlashWindow(Handle, True)
  end;
  //Frame Counter
  Inc(Frames);
  if GetTickCount - StartTick >=500 then
  begin
    Caption := Format('http://www.delphigl.com/ - %s [%f]', [Projektname, Frames/(GetTickCount - StartTick)*1000]);
    Frames := 0;
    StartTick := GetTickCount
  end;
  //Und anzeigen
  SwapBuffers(Canvas.Handle)
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  if not OpenGlInitialized then Exit;
  glViewport(0, 0, ClientWidth, ClientHeight);
  //Projektionsmatrix resetten
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //Perspektivische Darstellung
  gluPerspective(45.0,ClientWidth/ClientHeight,NearClipping,FarClipping);// Calculate The Aspect Ratio Of The Window
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  ShutdownGl
end;

end.
