unit template_init;

{******************************************************************************}
{                                                                              }
{       Borland Delphi Runtime Library                                         }
{       Mini Variante des DGL VCL Templates, um Demos klein zu halten          }
{       Modul zum initialisieren von OpenGl                                    }
{                                                                              }
{ Copyright (C) 2003 Nico Michaelis(Delphic)                                   }
{ All Rights Reserved.                                                         }
{                                                                              }
{       Obtained through:                                                      }
{       Delphi OpenGL Community(DGL)                                           }
{                                                                              }
{ You may retrieve the latest version of this file at the Delphi OpenGL        }
{ Community home page, located at http://www.delphigl.com/                     }
{                                                                              }
{ The contents of this file are used with permission, subject to               }
{ the Mozilla Public License Version 1.1 (the "License"); you may              }
{ not use this file except in compliance with the License. You may             }
{ obtain a copy of the License at                                              }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an                  }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or               }
{ implied. See the License for the specific language governing                 }
{ rights and limitations under the License.                                    }
{                                                                              }
{******************************************************************************}

interface
uses
  Windows, Dialogs, Forms, OpenGl12;
const
  //Entfernung der Clipping planes
  FarClipping   = 1000.0;
  NearClipping  = 1.0;

var
  OpenGLInitialized : Boolean; //Ist true, wenn OpenGL bereit zum zeichnen ist
  //Daten, fr den Framecounter
  StartTick : Cardinal;
  Frames : LongInt;

procedure InitGl(MainForm : TForm);
procedure ShutdownGl;

implementation
var
  RC : HGLRC;         //OpenGL Rendering Context
  fMainForm : tForm;

procedure InitGl(MainForm : TForm);
  procedure SetPixelFormat;
  var
    PixelFormat      : TGLuint;
    PFD              : pixelformatdescriptor;
  begin
    with pfd do
      begin
        nSize:= SizeOf( PIXELFORMATDESCRIPTOR );
        nVersion:= 1;
        dwFlags:= PFD_DRAW_TO_WINDOW
          or PFD_SUPPORT_OPENGL
          or PFD_DOUBLEBUFFER;
        iPixelType:= PFD_TYPE_RGBA;
        cColorBits:= 16;             //Farbtiefe
        cRedBits:= 0;
        cRedShift:= 0;
        cGreenBits:= 0;
        cBlueBits:= 0;
        cBlueShift:= 0;
        cAlphaBits:= 0;
        cAlphaShift:= 0;
        cAccumBits:= 0;              //Accumulation Buffer
        cAccumRedBits:= 0;
        cAccumGreenBits:= 0;
        cAccumBlueBits:= 0;
        cAccumAlphaBits:= 0;
        cDepthBits:= 16;             //Z-Buffer Tiefe
        cStencilBits:= 0;            //Stencil Buffer
        cAuxBuffers:= 0;
        iLayerType:= PFD_MAIN_PLANE;
        bReserved:= 0;
        dwLayerMask:= 0;
        dwVisibleMask:= 0;
        dwDamageMask:= 0
      end;
    //Pixel Format setzten
    PixelFormat := ChoosePixelFormat(MainForm.Canvas.Handle, @pfd);
    if (PixelFormat=0) then
      MessageDlg('Es konnte kein passendes PixelFormat gefunden werden.', mtError, [mbOk], 0);
    if (not Windows.SetPixelFormat(MainForm.Canvas.Handle,PixelFormat,@pfd)) then
      MessageDlg('PixelFormat konnte nicht gesetzt werden.', mtError, [mbOk], 0)
  end; (*SetPixelFormat*)

  procedure StartGl;
  begin
    //Rendering Context initialisieren
    RC := wglCreateContext(MainForm.Canvas.Handle);
    if (RC=0) then
    begin
      MessageDlg('Rendering Context kann nicht erstellt werden.', mtError, [mbOk], 0);
      Halt(100)
    end;

    if (not wglMakeCurrent(MainForm.Canvas.Handle, RC)) then
    begin
      MessageDlg('Rendering Context kann nicht aktiviert werden.', mtError, [mbOk], 0);
      Halt(100)
    end
  end; (*StartGl*)

  procedure SetupGL;
  begin
    //OpenGL Voreinstellungen
    glClearColor(0.0, 0.0, 0.0, 0.0); //Hintergrundfarbe
    glEnable(GL_DEPTH_TEST); //Tiefenpuffer aktivieren
    //glEnable(GL_CULL_FACE); //Backface Culling aktivieren

    glShadeModel(GL_SMOOTH)
  end; (*SetupGL*)

begin
  RC := 0;

  SetPixelFormat;
  StartGL;

  //GL Extensions neu auslesen
  ClearExtensions;
  ReadExtensions;

  SetupGL;

  //Kleinkram initialisieren
  Frames := 0;
  StartTick := GetTickCount;
  fMainForm := MainForm;
  //OpenGL ist nun bereit
  OpenGLInitialized := True
end; (*InitGl*)

procedure ShutdownGl;
begin
  OpenGLInitialized := False;
  if RC<>0 then
    begin
      if (not wglMakeCurrent(fMainForm.Canvas.Handle,0)) then
        MessageDlg('Rendering Context konnte nicht abgewhlt werden.', mtError, [mbOk], 0);
      if (not wglDeleteContext(RC)) then
        MessageDlg('Rendering Context konnte nicht freigegeben werden.', mtError, [mbOk], 0)
    end;
  RC:=0
end;


initialization
  OpenGLInitialized := False

end.
